import { continuedIndent, indentNodeProp, foldNodeProp, LRLanguage, } from "@codemirror/language";
import { styleTags, tags } from "@codemirror/highlight";
import { parser } from "lezer-tasl";
const styleNodeProp = styleTags({
    Comment: tags.comment,
    "NamespaceDefinition/Prefix": tags.name,
    "NamespaceDefinition/NamespaceURI": [tags.url, tags.link],
    "TypeDefinition/Name": tags.className,
    "ClassDeclaration/Term": tags.className,
    Type: tags.typeName,
    URI: tags.atom,
    "Literal/Term": tags.string,
    "OpenBrace CloseBrace": tags.brace,
    "OpenBracket CloseBracket": tags.bracket,
    "Component/Term Option/Term": tags.propertyName,
    "RightArrow LeftArrow": tags.separator,
    "Reference/Term": tags.className,
    "namespace type class": tags.keyword,
});
export const taslLanguage = LRLanguage.define({
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Product: continuedIndent({ except: /^\s*\}/ }),
                Coproduct: continuedIndent({ except: /^\s*\]/ }),
            }),
            foldNodeProp.add({
                Product(subtree) {
                    return { from: subtree.from + 1, to: subtree.to - 1 };
                },
                Coproduct(subtree) {
                    return { from: subtree.from + 1, to: subtree.to - 1 };
                },
            }),
            styleNodeProp,
        ],
    }),
    languageData: {
        closeBrackets: { brackets: ["[", "{"] },
        indentOnInput: /^\s*[\}\]]$/,
        commentTokens: { line: "#" },
    },
});
